@@ -72,6 +72,7 @@ findfile_in_dir(char *fnam, char *dir, int recdepth, char **suf) {
 	struct dirent *de;
 	char *ff, *fdir, *p, *q, **sp;
 	struct decompressor *dc;
+	int secondpass = 0;
 
 	ispipe = 0;
 
@@ -82,10 +83,16 @@ findfile_in_dir(char *fnam, char *dir, int recdepth, char **suf) {
 	} else
 		fdir = 0;		/* just to please gcc */
 
+	/* Scan the directory twice: first for files, then
+	   for subdirectories, so that we do never search
+	   a subdirectory when the directory itself already
+	   contains the file we are looking for. */
+ StartScan:
 	d = opendir(dir);
 	if (d == NULL)
 	    return NULL;
 	while ((de = readdir(d)) != NULL) {
+	    struct stat statbuf;
 	    int okdir;
 
 	    if (strcmp(de->d_name, ".") == 0 ||
@@ -97,8 +104,7 @@ findfile_in_dir(char *fnam, char *dir, int recdepth, char **suf) {
 
 	    okdir = (ff && strcmp(de->d_name, fdir) == 0);
 
-	    if (recdepth || okdir) {
-       		struct stat statbuf;
+	    if ((secondpass && recdepth) || okdir) {
 		char *a;
 
 		a = xmalloc(strlen(dir) + strlen(de->d_name) + 2);
@@ -115,6 +121,9 @@ findfile_in_dir(char *fnam, char *dir, int recdepth, char **suf) {
 		free(a);
 	    }
 
+	    if (secondpass)
+		    continue;
+
 	    /* Should we be in a subdirectory? */
 	    if (ff)
 		    continue;
@@ -127,6 +136,8 @@ findfile_in_dir(char *fnam, char *dir, int recdepth, char **suf) {
 		    continue;
 
 	    sprintf(pathname, "%s/%s", dir, de->d_name);
+	    if (stat(pathname, &statbuf) != 0 || !S_ISREG(statbuf.st_mode))
+		    continue;
 
 	    /* Does tail consist of a known suffix and possibly
 	       a compression suffix? */
@@ -144,6 +155,11 @@ findfile_in_dir(char *fnam, char *dir, int recdepth, char **suf) {
 		    }
 	    }
 	}
+	closedir(d);
+	if (recdepth > 0 && !secondpass) {
+		secondpass = 1;
+		goto StartScan;
+	}
 	return NULL;
 }
 
@@ -166,13 +182,16 @@ FILE *findfile(char *fnam, char **dirpath, char **suffixes) {
 	/* Test for full pathname - opening it failed, so need suffix */
 	/* (This is just nonsense, for backwards compatibility.) */
 	if (*fnam == '/') {
+	    struct stat statbuf;
+
 	    for (sp = suffixes; *sp; sp++) {
 		if (strlen(fnam) + strlen(*sp) + 1 > sizeof(pathname))
 		    continue;
 		if (*sp == 0)
 		    continue;	/* we tried it already */
 		sprintf(pathname, "%s%s", fnam, *sp);
-		if((fp = fopen(pathname, "r")) != NULL)
+		if(stat(pathname, &statbuf) == 0 && S_ISREG(statbuf.st_mode)
+		   && (fp = fopen(pathname, "r")) != NULL)
 		    return fp;
 	    }
 
@@ -182,7 +201,9 @@ FILE *findfile(char *fnam, char **dirpath, char **suffixes) {
 			+ strlen(dc->ext) + 1 > sizeof(pathname))
 			    continue;
 		    sprintf(pathname, "%s%s%s", fnam, *sp, dc->ext);
-		    if ((fp = fopen(pathname, "r")) != NULL) {
+		    if (stat(pathname, &statbuf) == 0
+			&& S_ISREG(statbuf.st_mode)
+			&& (fp = fopen(pathname, "r")) != NULL) {
 			    fclose(fp);
 			    return pipe_open(dc);
 		    }
