@@ -29,12 +29,13 @@ extern char *progname;
 extern int force;
 
 static char *unidirpath[] = { "", DATADIR "/" UNIMAPDIR "/", 0 };
-static char *unisuffixes[] = { "", ".uni", 0 };
+static char *unisuffixes[] = { "", ".uni", ".sfm", 0 };
 
 #ifdef MAIN
 #include "version.h"
 int verbose = 0;
 int force = 0;
+int debug = 0;
 
 static void
 usage(void) {
@@ -53,8 +54,8 @@ main(int argc, char *argv[]) {
 	set_progname(argv[0]);
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+	textdomain(PACKAGE_NAME);
 
 	if (argc == 2 &&
 	    (!strcmp(argv[1], "-V") || !strcmp(argv[1], "--version")))
@@ -73,18 +74,18 @@ main(int argc, char *argv[]) {
 		}
 	}
 
-	if (argc > 2 || (argc == 1 && !outfnam))
+	if (argc > optind+1 || (argc == optind && !outfnam))
 		usage();
 
 	fd = getfd(console);
 
 	if (outfnam) {
 		saveunicodemap(fd, outfnam);
-		if (argc == 1)
+		if (argc == optind)
 			exit(0);
 	}
 
-	if (argc == 2)
+	if (argc == optind+1)
 		infnam = argv[optind];
 	loadunicodemap(fd, infnam);
 	exit(0);
@@ -345,13 +346,29 @@ appendunicodemap(int fd, FILE *fp, int fontsize, int utf8) {
 	descr = getunicodemap(fd);
 	list = descr.entries;
 
+		
 	for(i=0; i<fontsize; i++) {
-		for(j=0; j<descr.entry_ct; j++)
+#if 0
+		/* More than one mapping is not a sequence! */
+		int no = 0;
+		for(j=0; j<descr.entry_ct; j++) 
 			if (list[j].fontpos == i)
+				no++;
+		if (no > 1)
+			appendseparator(fp, 1, utf8);
+#endif		
+		if (debug) printf ("\nchar %03x: ", i);
+		for(j=0; j<descr.entry_ct; j++)
+			if (list[j].fontpos == i) {
+				if (debug)
+					printf ("%04x ", list[j].unicode);
 				appendunicode(fp, list[j].unicode, utf8);
+			}
 		appendseparator(fp, 0, utf8);
 	}
 
+
+	if (debug) printf ("\n");
 	if (verbose)
 		printf(_("Appended Unicode map\n"));
 }
