@@ -4,7 +4,7 @@
 #include <stdlib.h>	/* exit */
 #include "psf.h"
 
-void
+static void
 usage(void) {
 	fprintf(stderr, "call: outpsfheader psftype fontsize charsize hastable\n");
 	exit(1);
