@@ -16,10 +16,10 @@
  * Default keymap, and where the kernel copy of it lives.
  */
 #ifdef __sparc__
-# define DEFMAP "sunkeymap.map"
+# define DEFMAP "sunkeymap.kmap"
 # define KERNDIR "/usr/src/linux/drivers/sbus/char"
 #else
-# define DEFMAP "defkeymap.map"
+# define DEFMAP "defkeymap.kmap"
 # define KERNDIR "/usr/src/linux/drivers/char"
 #endif
 
@@ -28,3 +28,4 @@ extern char pathname[];
 extern void fpclose(FILE *fp);
 
 extern int verbose;
+extern int debug;
