@@ -121,8 +121,8 @@ main(int argc, char **argv) {
     set_progname(argv[0]);
 
     setlocale(LC_ALL, "");
-    bindtextdomain(PACKAGE, LOCALEDIR);
-    textdomain(PACKAGE);
+    bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+    textdomain(PACKAGE_NAME);
 
     if (argc < 2)
       usage();
@@ -157,7 +157,7 @@ main(int argc, char **argv) {
  	fpclose(fin);
     }
 
-    fd = getfd();
+    fd = getfd(NULL);
 
     if(ioctl(fd, TIOCGWINSZ, &winsize)) {
 	perror("TIOCGWINSZ");
