@@ -19,7 +19,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <sys/kd.h>
+#include <linux/kd.h>
 #include <endian.h>
 #include <sysexits.h>
 #include "paths.h"
@@ -43,11 +43,12 @@ static void loadnewfonts(int fd, char **ifiles, int ifilct,
 			int iunit, int hwunit, int no_m, int no_u);
 extern void saveoldmap(int fd, char *omfil);
 extern void loadnewmap(int fd, char *mfil);
-extern void activatemap(void);
-extern void disactivatemap(void);
+extern void activatemap(int fd);
+extern void disactivatemap(int fd);
 
 int verbose = 0;
 int force = 0;
+int debug = 0;
 
 /* search for the font in these directories (with trailing /) */
 char *fontdirpath[] = { "", DATADIR "/" FONTDIR "/", 0 };
@@ -72,14 +73,14 @@ usage(void)
         fprintf(stderr, _(
 "Usage: setfont [write-options] [-<N>] [newfont..] [-m consolemap] [-u unicodemap]\n"
 "  write-options (take place before file loading):\n"
-"    -o  <filename>	Write current font to <filename>\n"
-"    -O  <filename>	Write current font and unicode map to <filename>\n"
-"    -om <filename>	Write current consolemap to <filename>\n"
-"    -ou <filename>	Write current unicodemap to <filename>\n"
+"    -o  <filename>  Write current font to <filename>\n"
+"    -O  <filename>  Write current font and unicode map to <filename>\n"
+"    -om <filename>  Write current consolemap to <filename>\n"
+"    -ou <filename>  Write current unicodemap to <filename>\n"
 "If no newfont and no -[o|O|om|ou|m|u] option is given,\n"
 "a default font is loaded:\n"
-"    setfont             Load font \"default[.gz]\"\n"
-"    setfont -<N>        Load font \"default8x<N>[.gz]\"\n"
+"    setfont         Load font \"default[.gz]\"\n"
+"    setfont -<N>    Load font \"default8x<N>[.gz]\"\n"
 "The -<N> option selects a font from a codepage that contains three fonts:\n"
 "    setfont -{8|14|16} codepage.cp[.gz]   Load 8x<N> font from codepage.cp\n"
 "Explicitly (with -m or -u) or implicitly (in the fontfile) given mappings\n"
@@ -87,12 +88,13 @@ usage(void)
 "    -h<N>      (no space) Override font height.\n"
 "    -m <fn>    Load console screen map.\n"
 "    -u <fn>    Load font unicode map.\n"
-"    -m none	Suppress loading and activation of a screen map.\n"
-"    -u none	Suppress loading of a unicode map.\n"
-"    -v		Be verbose.\n"
-"    -V		Print version and exit.\n"
-"Files are loaded from the current directory or /usr/lib/kbd/*/.\n"
-));
+"    -m none    Suppress loading and activation of a screen map.\n"
+"    -u none    Suppress loading of a unicode map.\n"
+"    -v         Be verbose.\n"
+"    -C <cons>  Indicate console device to be used.\n"
+"    -V         Print version and exit.\n"
+"Files are loaded from the current directory or %s/*/.\n"
+), DATADIR);
 	exit(EX_USAGE);
 }
 
@@ -101,22 +103,24 @@ usage(void)
 int
 main(int argc, char *argv[]) {
 	char *ifiles[MAXIFILES];
-	char *mfil, *ufil, *Ofil, *ofil, *omfil, *oufil;
+	char *mfil, *ufil, *Ofil, *ofil, *omfil, *oufil, *console;
 	int ifilct = 0, fd, i, iunit, hwunit, no_m, no_u;
 	int restore = 0;
 
 	set_progname(argv[0]);
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+	textdomain(PACKAGE_NAME);
 
-	fd = getfd();
-
-	ifiles[0] = mfil = ufil = Ofil = ofil = omfil = oufil = 0;
+	ifiles[0] = mfil = ufil = Ofil = ofil = omfil = oufil = NULL;
 	iunit = hwunit = 0;
 	no_m = no_u = 0;
+	console = NULL;
 
+	/*
+	 * No getopt() here because of the -om etc options.
+	 */
 	for (i = 1; i < argc; i++) {
 	    if (!strcmp(argv[i], "-V")) {
 		print_version_and_exit();
@@ -124,6 +128,10 @@ main(int argc, char *argv[]) {
 	        verbose++;
 	    } else if (!strcmp(argv[i], "-R")) {
 		restore = 1;
+	    } else if (!strcmp(argv[i], "-C")) {
+		if (++i == argc || console)
+		  usage();
+		console = argv[i];
 	    } else if (!strcmp(argv[i], "-O")) {
 		if (++i == argc || Ofil)
 		  usage();
@@ -179,6 +187,21 @@ main(int argc, char *argv[]) {
 	    exit(EX_USAGE);
 	}
 
+	fd = getfd(console);
+
+	int kd_mode = -1;
+	if (!ioctl(fd, KDGETMODE, &kd_mode) && (kd_mode == KD_GRAPHICS))
+	  {
+	    /*
+	     * PIO_FONT will fail on a console which is in foreground and in KD_GRAPHICS mode.
+	     * 2005-03-03, jw@suse.de.
+	     */
+	    if (verbose)
+	      printf("setfont: graphics console %s skipped\n", console?console:"");
+	    close(fd);
+	    return 0;
+	  }
+
 	if (!ifilct && !mfil && !ufil &&
 	    !Ofil && !ofil && !omfil && !oufil && !restore)
 	  /* reset to some default */
@@ -198,7 +221,7 @@ main(int argc, char *argv[]) {
 
 	if (mfil) {
 	    loadnewmap(fd, mfil);
-	    activatemap();
+	    activatemap(fd);
 	    no_m = 1;
 	}
 
@@ -225,7 +248,7 @@ static int erase_mode = 1;
 static void
 do_loadfont(int fd, char *inbuf, int width, int height, int hwunit,
 	    int fontsize, char *pathname) {
-	char *buf;
+	unsigned char *buf;
 	int i, buflen;
 	int bytewidth = (width+7)/8;
 	int charsize = height * bytewidth;
@@ -322,15 +345,28 @@ do_loadtable(int fd, struct unicode_list *uclistheads, int fontsize) {
 	up = xmalloc(maxct * sizeof(struct unipair));
 	for (i = 0; i < fontsize; i++) {
 		ul = uclistheads[i].next;
+		if (debug) printf ("char %03x:", i);
 		while(ul) {
 			us = ul->seq;
 			if (us && ! us->next) {
 				up[ct].unicode = us->uc;
 				up[ct].fontpos = i;
 				ct++;
+				if (debug) printf (" %04x", us->uc);
 			}
+			else
+				if (debug) {
+					printf (" seq: <");
+					while (us) {
+						printf (" %04x", us->uc);
+						us = us->next;
+					}
+					printf (" >");
+				}
 			ul = ul->next;
+			if (debug) printf (",");
 		}
+		if (debug) printf ("\n");
 	}
 	if (ct != maxct) {
 		char *u = _("%s: bug in do_loadtable\n");
@@ -389,8 +425,10 @@ loadnewfonts(int fd, char **ifiles, int ifilct,
 			fprintf(stderr, _("When loading several fonts, all "
 					  "must be psf fonts - %s isn't\n"),
 				pathname);
+			fpclose(fpi);
 			exit(EX_DATAERR);
 		}
+		fpclose(fpi);		// avoid zombies, jw@suse.de (#88501)
 		bytewidth = (width+7) / 8;
 		height = fontbuflth / (bytewidth * fontsize);
 		if (verbose)
@@ -474,6 +512,7 @@ loadnewfont(int fd, char *ifil, int iunit, int hwunit, int no_m, int no_u) {
 	if(readpsffont(fpi, &inbuf, &inputlth, &fontbuf, &fontbuflth,
 		       &width, &fontsize, 0,
 		       no_u ? NULL : &uclistheads) == 0) {
+		fpclose(fpi);
 		/* we've got a psf font */
 		bytewidth = (width+7) / 8;
 		height = fontbuflth / (bytewidth * fontsize);
@@ -488,6 +527,7 @@ loadnewfont(int fd, char *ifil, int iunit, int hwunit, int no_m, int no_u) {
 #endif
 		return;
 	}
+	fpclose(fpi);		// avoid zombies, jw@suse.de (#88501)
 
 	/* instructions to combine fonts? */
 	{ char *combineheader = "# combine partial fonts\n";
@@ -599,7 +639,7 @@ do_saveoldfont(int fd, char *ofil, FILE *fpo, int unimap_follows,
 
 /* this is the max font size the kernel is willing to handle */
 #define MAXFONTSIZE	65536
-	char buf[MAXFONTSIZE];
+	unsigned char buf[MAXFONTSIZE];
 
 	int i, ct, width, height, bytewidth, charsize, kcharsize;
 
@@ -681,12 +721,19 @@ saveoldfontplusunicodemap(int fd, char *Ofil) {
    probably it should copy the default from the current console?
    But what if we want a new one because the current one is messed up? */
 /* For the moment: only the current console, only the G0 set */
-void
-activatemap(void) {
-    printf("\033(K");
+
+static void
+send_escseq(int fd, char *seq, int n) {
+	if (write(fd, seq, n) != n)  /* maybe fd is read-only */
+		printf(seq);
 }
 
 void
-disactivatemap(void) {
-    printf("\033(B");
+activatemap(int fd) {
+	send_escseq(fd, "\033(K", 3);
+}
+
+void
+disactivatemap(int fd) {
+	send_escseq(fd, "\033(B", 3);
 }
