@@ -14,6 +14,10 @@
 #include "nls.h"
 #include "version.h"
 
+#ifdef __sparc__
+#include <asm/kbio.h>
+#endif
+
 static void
 usage(void)
 {
@@ -152,8 +156,8 @@ main(int argc, char **argv) {
     set_progname(argv[0]);
 
     setlocale(LC_ALL, "");
-    bindtextdomain(PACKAGE, LOCALEDIR);
-    textdomain(PACKAGE);
+    bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+    textdomain(PACKAGE_NAME);
 
     if (argc == 2 && (!strcmp("-V", argv[1]) || !strcmp("--version", argv[1])))
 	print_version_and_exit();
